#!/bin/bash
#------------------------------------------------------------------------------
# This is a script for transferring MAS data from a supercomputer to PSI.
#
# - Usage: push_mas.sh <run_name> [seq0] [seq1] [dseq] [flags]
#
# - See the flag meaning/defaults by running the script with no arguments.
#
# - It uses the shflags parsing library, so you need to put that in a place 
#   this script can find it (see header).
#
# - Currently the source and destination run folders are hardcoded.
#   - edit these to match your current setup
#
# - TBD: implementing the silo and restart functionality.
#   - currently use -A instead...
#
# - Developed by CD sometime in 2015
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Header with settings for parsing
#------------------------------------------------------------------------------
# source the shflags parsing library
. ${HOME}/sh_scripts/shflags-1.0.3/src/shflags

# define flags
DEFINE_boolean 'all' false 'copy the entire directory 1 to 1' 'A'
DEFINE_boolean 'hdf' false 'copy hdf files for standard mhd fields' 'd'
DEFINE_boolean 'rst' false 'copy restart matching the sequence number' 'r'
DEFINE_boolean 'silo' false 'make and copy silo files' 's'
DEFINE_boolean 'vvec' false 'copy the velocity vector field' 'v'
DEFINE_boolean 'avec' false 'copy the vector potential field' 'a'
DEFINE_boolean 'bvec' false 'copy the magnetic vector field' 'b'
DEFINE_boolean 'jvec' false 'copy the current vector field' 'j'
DEFINE_boolean 'emis' false 'copy rho and t only' 'e'
DEFINE_boolean 'wave' false 'copy wave variables (e[p,m], z[p,m])' 'z'
DEFINE_boolean 'heat' false 'copy heat' 't'
DEFINE_boolean 'tracers' false 'copy the tracer dumps' 'T'
DEFINE_string  'vars' "none" 'comma separated list of specifc vars'
DEFINE_boolean 'links' false 'follow links' 'L'
DEFINE_boolean 'dryrun' false 'use rsync in "dry-run" mode' 'n'
DEFINE_boolean 'progress' false 'add the --progress option to rsync mode' 'p'

# define other defaults
notset=-777
seq0=${notset}
seq1=${notset}
dseq=1

cname="PUSH_MAS"
cvers="1.00"
cdate="10/20/2015"

# define the usage message
FLAGS_HELP=`cat << EOF

 ### ${cname} Version ${cvers} of ${cdate}
 ### Copy a MAS run to a local machine using rsync
 
 - By default it will copy just the log/input files. But the
   flags can force it to copy hdf data (or subsets) as well
   as make/copy silo files.

 - Note that the destination and silo script are hardcoded

USAGE: $0 runname [seq0] [seq1] [dseq] [flags]
EOF`

# This function prints the help and exits with an error
die_help()
{
  [ $# -gt 0 ] && echo "error: $@" >&2
  flags_help
  echo ""
  exit 1
}

# This function prints an error message and exits with status 1
die_message()
{
  echo ""
  echo "### ERROR! ${cname}"
  echo "### $1"
  echo ""
  exit 1
}

# parse the command-line
FLAGS "$@" || exit 1
eval set -- "${FLAGS_ARGV}"


# Check positional command line arguments: (if nothing set, print help and quit)
if [ "$#" -lt "1" ]; then
   die_help
fi 

# change the behavior depending on the number of positional arguments
if [ "$#" -eq "1" ]; then
   runname=$1
fi 
if [ "$#" -eq "2" ]; then
   runname=$1
   seq0=$2
   seq1=$seq0
fi 
if [ "$#" -eq "3" ]; then
   runname=$1
   seq0=$2
   seq1=$3
fi 
if [ "$#" -eq "4" ]; then
   runname=$1
   seq0=$2
   seq1=$3
   dseq=$4
fi 

#------------------------------------------------------------------------------
# Hardcoded settings
#------------------------------------------------------------------------------

# this is where runs are saved on Pleiades
workdir="/nobackupp8/cjdowns/mas/runs"

# Set the base directory on downstown where the data will go
server="downstown.predsci.com"
username="cdowns"
# DOWNSTOWN LOCATIONS:
basedir="/Volumes/datadisk3/mas/runs/Eclipse2019"

sbase="${username}@${server}:${basedir}"

# default rsync
rsync_flags="-avh"


#------------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------------

rundir="${workdir}/${runname}"

# check if the run directory exists
if [ ! -d $rundir ]; then
   die_message "$rundir is not a dir or does not exist!"
fi

# check if we are following links
if [ ${FLAGS_links} -eq ${FLAGS_TRUE} ]; then
   rsync_flags="${rsync_flags}L"
fi

# check if it is a dryrun
if [ ${FLAGS_dryrun} -eq ${FLAGS_TRUE} ]; then
   rsync_flags="${rsync_flags}n"
fi

# check if we have the progress flag
if [ ${FLAGS_progress} -eq ${FLAGS_TRUE} ]; then
   rsync_flags="--progress ${rsync_flags}"
fi

#------------------------------------------------------------------------------
# determine if any hdf files are desired to be transferred
#------------------------------------------------------------------------------
copy_data=${FLAGS_FALSE}
if [ ${FLAGS_hdf} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_vvec} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_avec} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_bvec} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_jvec} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_emis} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_heat} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_wave} -eq ${FLAGS_TRUE} ] || \
   [ ${FLAGS_vars} != "none" ]; then
   copy_data=${FLAGS_TRUE}
fi

copy_tracers=${FLAGS_FALSE}
if [ ${FLAGS_tracers} -eq ${FLAGS_TRUE} ]; then
   copy_tracers=${FLAGS_TRUE}
fi

#if [ ${copy_data} -eq ${FLAGS_TRUE} ]; then
#   echo "copy data IS SET"
#fi

#------------------------------------------------------------------------------
# determine the sequence numbers (hdf, silo, or restart)
#------------------------------------------------------------------------------

# cannot do rst and data sequences at the same time
if [ ${FLAGS_rst} -eq ${FLAGS_TRUE} ] && [ ${copy_data} -eq ${FLAGS_TRUE} ]; then
   die_message "Cannot copy restarts and sequences at the same time (use all instead)."
fi

# if copying restarts without setting a sequence, take the final name
if [ ${FLAGS_rst} -eq ${FLAGS_TRUE} ] && [ $seq0 -eq ${notset} ]; then
   seq0=""
fi

# if copying data without setting a sequence, determine the sequences using get_dumps
if [ ${copy_data} -eq ${FLAGS_TRUE} ] && [ $seq0 -eq ${notset} ]; then
   tmpfile="tmp_hdf_times.txt"
   ofile="${rundir}/o${runname}"
   if [ ! -f ${ofile} ]; then
      die_message "${ofile} cannot be found"
   fi
   get_dumps ${rundir}/o${runname} | awk /seq/ | pw 4 8 > ${tmpfile}
   seq0=`head -n 1 ${tmpfile} | pw 1`
   seq1=`tail -n 1 ${tmpfile} | pw 1`
   \rm ${tmpfile}
fi

# just copy all of the particle dumps for simplicity
if [ ${copy_tracers} -eq ${FLAGS_TRUE} ] ; then
   echo "YO!"
   tmpfile="tmp_tracer_times.txt"
   ofile="${rundir}/o${runname}"
   if [ ! -f ${ofile} ]; then
      die_message "${ofile} cannot be found"
   fi
   get_particle_dumps ${rundir}/o${runname} | awk /seq/ | pw 4 8 > ${tmpfile}
   pseq0=`head -n 1 ${tmpfile} | pw 1`
   pseq1=`tail -n 1 ${tmpfile} | pw 1`
   \rm ${tmpfile}
   dpseq=1
   pseqs=()
   if [ ! -z $pseq0 ]; then
      num0=`seq2num $pseq0`
      num1=`seq2num $pseq1`
      for num in `seq $num0 $dpseq $num1`; do
         pseq=`num2seq $num`
         pseqs+=("$pseq")
      done
   fi
fi

# get an array with EVERY sequence number
if [ ${FLAGS_rst} -eq ${FLAGS_TRUE} ] || [ ${copy_data} -eq ${FLAGS_TRUE} ]; then
   seqs=()
   if [ ! -z $seq0 ]; then
      num0=`seq2num $seq0`
      num1=`seq2num $seq1`
      for num in `seq $num0 $dseq $num1`; do
         seq=`num2seq $num`
         seqs+=("$seq")
      done
   fi
fi


# if the hdf flag is set, copy the standard variables
if [ ${FLAGS_hdf} -eq ${FLAGS_TRUE} ]; then
   FLAGS_vvec=${FLAGS_TRUE}
   FLAGS_bvec=${FLAGS_TRUE}
   FLAGS_jvec=${FLAGS_TRUE}
   FLAGS_emis=${FLAGS_TRUE}
fi

echo ""
echo "### ${cname}: Beginning Transfer..."
echo "### From: ${rundir}"
echo "### To: ${basedir}/${runname}"
echo "### Server: ${server}"
echo ""

#------------------------------------------------------------------------------
# start copying things
#------------------------------------------------------------------------------
# copy the input/log files (this also creates the directory)
echo "### Transferring logfiles ${runname}"
rsync $rsync_flags \
   --exclude="*.h5" \
   --exclude="*.hdf" \
   --exclude="mas" \
   --exclude="mas.f" \
   --exclude="silo" \
   --exclude="submit_mas" \
   ${rundir} ${sbase}/
if [ $? -ne 0 ]; then
   die_message "logfile rsync FAILED for some reason! Exiting."
fi
echo ""

# define a general rsync function (to push/pop so the input list stays small)
rsync_filelist()
{
   #echo input: $*
   pushd $rundir "$@" > /dev/null
   rsync $rsync_flags $* ${sbase}/${runname}/
   popd "$@" > /dev/null
}

#------------------------------------------------------------------------------
# copy the hdf var files
#------------------------------------------------------------------------------
if [ ${copy_data} -eq ${FLAGS_TRUE} ]; then
   # build up the variable list
   varlist=""
   if [ ${FLAGS_emis} -eq ${FLAGS_TRUE} ]; then
      varlist+=( rho t )
   fi
   if [ ${FLAGS_vvec} -eq ${FLAGS_TRUE} ]; then
      varlist+=( vr vt vp )
   fi
   if [ ${FLAGS_avec} -eq ${FLAGS_TRUE} ]; then
      varlist+=( ar at ap )
   fi
   if [ ${FLAGS_bvec} -eq ${FLAGS_TRUE} ]; then
      varlist+=( br bt bp )
   fi
   if [ ${FLAGS_jvec} -eq ${FLAGS_TRUE} ]; then
      varlist+=( jr jt jp )
   fi
   if [ ${FLAGS_heat} -eq ${FLAGS_TRUE} ]; then
      varlist+=( heat )
   fi
   if [ ${FLAGS_wave} -eq ${FLAGS_TRUE} ]; then
      varlist+=( ep em zp zm )
   fi
   if [ ${FLAGS_vars} != "none" ]; then
      split=$(echo $FLAGS_vars | tr "," "\n")
      for var in $split; do
         varlist+=( $var )
      done
   fi

   echo "### Transferring vars: ${varlist[@]}"
   echo "### For sequences: ${seq0} to ${seq1}"
   echo ""

   #for var in ${varlist[@]}; do
   #   echo var i: $var
   #done

   filelist=""
   for seq in ${seqs[@]}; do
      for var in ${varlist[@]}; do
         filelist+=( ${var}${seq}.hdf )
      done
   done
   rsync_filelist "${filelist[@]}"
fi

#------------------------------------------------------------------------------
# Copy tracer particles if necessary
#------------------------------------------------------------------------------
if [ ${copy_tracers} -eq ${FLAGS_TRUE} ]; then
   # build up the variable list
   varlist=""
   if [ ${FLAGS_tracers} -eq ${FLAGS_TRUE} ]; then
      varlist+=( tracers_pos tracers_ds )
   fi

   echo "### Transferring tracers: ${varlist[@]}"
   echo "### For all sequences"
   echo ""

   filelist=""
   for seq in ${pseqs[@]}; do
      for var in ${varlist[@]}; do
         filelist+=( ${var}${seq}.hdf )
      done
   done
   rsync_filelist "${filelist[@]}"
fi

# I GOT TIRED! silo and restart functionality TBD

#------------------------------------------------------------------------------
# Optionally copy eveything
#------------------------------------------------------------------------------
if [ ${FLAGS_all} -eq ${FLAGS_TRUE} ]; then
	echo "### Copying the entire run directory!"

   rsync $rsync_flags \
      ${rundir} ${sbase}/
   if [ $? -ne 0 ]; then
      die_message "logfile rsync FAILED for some reason! Exiting."
   fi
fi
echo ""



